//+------------------------------------------------------------------+
//|                                                 Wilder's DMI.mq4 |
//|                                                  coded by mladen |
//|                                                                  |
//| Directional movement index was developed by Welles Wilder        |
//+------------------------------------------------------------------+
#property copyright "mladen"
#property link      "mladenfx@gmail.com"

#property indicator_separate_window
#property  indicator_buffers 5
#property indicator_minimum  0
#property indicator_color1   clrDeepSkyBlue
#property indicator_color2   clrHotPink
#property indicator_color3   clrSandyBrown
#property indicator_color4   clrHotPink
#property indicator_color5   clrDimGray
#property indicator_width3   2 
#property indicator_width4   2 
#property indicator_style5   STYLE_DOT

//
//
//
//
//

extern ENUM_TIMEFRAMES TimeFrame    = PERIOD_CURRENT;
extern int             DMILength    =    14;
extern bool            ShowADX      =  true;
extern bool            ShowADXR     = false;
extern int             Level        =    25;
extern int             Smooth       =     5;
extern ENUM_MA_METHOD  SmoothMethod = MODE_EMA;
extern bool            Interpolate  = true;

//
//
//
//
//

double DIp[];
double DIm[];
double ADX[];
double ADXR[];
double ADXLevel[];
double averageDIp[];
double averageDIm[];
double averageTR[];

//
//
//
//
//

string indicatorFileName;
bool   returnBars;

//+------------------------------------------------------------------+
//|                                                                  |
//+------------------------------------------------------------------+
//
//
//
//

int init()
{
   IndicatorBuffers(8);
   SetIndexBuffer(0,DIp);      SetIndexLabel(0,"DI+");
   SetIndexBuffer(1,DIm);      SetIndexLabel(1,"DI-");
   SetIndexBuffer(2,ADX);      SetIndexLabel(2,"ADX");
   SetIndexBuffer(3,ADXR);     SetIndexLabel(3,"ADXR");
   SetIndexBuffer(4,ADXLevel); SetIndexLabel(4,NULL);
   SetIndexBuffer(5,averageDIp);
   SetIndexBuffer(6,averageDIm);
   SetIndexBuffer(7,averageTR);
      for (int i=0;i<8;i++) SetIndexEmptyValue(i,0.00);

   //
   //
   //
   //
   //

      indicatorFileName = WindowExpertName();
      returnBars        = (TimeFrame==-99);
      TimeFrame         = MathMax(TimeFrame,_Period);
      Smooth            = MathMax(Smooth,1);

   //
   //
   //
   //
   //
   
   IndicatorShortName(timeFrameToString(TimeFrame)+" Wilder\'s DMI ("+DMILength+")");
   return(0);
}

int deinit()
{
   return(0);
}


//+------------------------------------------------------------------+
//|                                                                  |
//+------------------------------------------------------------------+
//
//
//
//
//

int start()
{
   int counted_bars=IndicatorCounted();
   int i,limit;

   if(counted_bars<0) return(-1);
   if(counted_bars>0) counted_bars--;
           limit=MathMin(Bars-counted_bars,Bars-2);
           if (returnBars)  { DIp[0] = limit+1; return(0); }

   //
   //
   //
   //
   //
   
   if (TimeFrame == Period())
   {
      double sf = (DMILength-1.0)/DMILength;
      double arrc[3],arrp[2];
      for (i=limit;i>=0;i--)
      {
         arrc[0] = iMA(NULL,0,Smooth,0,SmoothMethod,PRICE_HIGH ,i  );
         arrc[1] = iMA(NULL,0,Smooth,0,SmoothMethod,PRICE_LOW  ,i  );
         arrc[2] = iMA(NULL,0,Smooth,0,SmoothMethod,PRICE_CLOSE,i+1);
         arrp[0] = iMA(NULL,0,Smooth,0,SmoothMethod,PRICE_HIGH ,i+1);
         arrp[1] = iMA(NULL,0,Smooth,0,SmoothMethod,PRICE_LOW  ,i+1);
         ArraySort(arrc); ArraySort(arrp); 
         double currTR  = MathMax(arrc[2],arrc[1])-MathMin(arrc[0],arrc[1]);
         double DeltaHi = arrc[2] - arrp[1];
	      double DeltaLo = arrp[0] - arrc[0];
         double plusDM  = 0.00;
         double minusDM = 0.00;
         
            if ((DeltaHi > DeltaLo) && (DeltaHi > 0)) plusDM  = DeltaHi;
            if ((DeltaLo > DeltaHi) && (DeltaLo > 0)) minusDM = DeltaLo;      
         
         //
         //
         //
         //
         //
         
            averageDIp[i] = sf*averageDIp[i+1] + plusDM;
            averageDIm[i] = sf*averageDIm[i+1] + minusDM;
            averageTR[i]  = sf*averageTR[i+1]  + currTR;
            ADXLevel[i]   = Level;

         //
         //
         //
         //
         //
                  
            DIp[i] = 0.00;                   
            DIm[i] = 0.00;                   
            if (averageTR[i] > 0)
               {              
                  DIp[i] = 100.00 * averageDIp[i]/averageTR[i];
                  DIm[i] = 100.00 * averageDIm[i]/averageTR[i];
               }            

            if(ShowADX)
               {
                  double DX;
                  if((DIp[i] + DIm[i])>0) 
                       DX = 100*MathAbs(DIp[i] - DIm[i])/(DIp[i] + DIm[i]); 
                  else DX = 0.00;
                  ADX[i] = sf*ADX[i+1] + DX/DMILength; 
                  if(ShowADXR)
                         ADXR[i] = 0.5*(ADX[i] + ADX[i+DMILength]);
               }
      }   
      return(0);      
   }
   
   //
   //
   //
   //
   //
   
   limit = MathMax(limit,MathMin(Bars,iCustom(NULL,TimeFrame,indicatorFileName,-99,0,0)*TimeFrame/Period()));
   for (i=limit;i>=0;i--)
   {
      int y = iBarShift(NULL,TimeFrame,Time[i]);
         ADXLevel[i] = Level;
         DIp[i]      = iCustom(NULL,TimeFrame,indicatorFileName,PERIOD_CURRENT,DMILength,ShowADX,ShowADXR,Level,Smooth,SmoothMethod,0,y);
         DIm[i]      = iCustom(NULL,TimeFrame,indicatorFileName,PERIOD_CURRENT,DMILength,ShowADX,ShowADXR,Level,Smooth,SmoothMethod,1,y);
            if (ShowADX)  ADX[i]  = iCustom(NULL,TimeFrame,indicatorFileName,PERIOD_CURRENT,DMILength,ShowADX,ShowADXR,Level,Smooth,SmoothMethod,2,y);
            if (ShowADXR) ADXR[i] = iCustom(NULL,TimeFrame,indicatorFileName,PERIOD_CURRENT,DMILength,ShowADX,ShowADXR,Level,Smooth,SmoothMethod,3,y);

            //
            //
            //
            //
            //
      
            if (!Interpolate || y==iBarShift(NULL,TimeFrame,Time[i-1])) continue;

            //
            //
            //
            //
            //

            datetime time = iTime(NULL,TimeFrame,y);
               for(int n = 1; i+n < Bars && Time[i+n] >= time; n++) continue;	
               for(int k = 1; k < n; k++)
               {
                  DIp[i+k] = DIp[i] + (DIp[i+n]-DIp[i])*k/n;
                  DIm[i+k] = DIm[i] + (DIm[i+n]-DIm[i])*k/n;
                     if (ShowADX)  ADX[i+k]  = ADX[i]  + (ADX[i+n] -ADX[i] )*k/n;
                     if (ShowADXR) ADXR[i+k] = ADXR[i] + (ADXR[i+n]-ADXR[i])*k/n;
               }
   }
   return(0);
}

//-------------------------------------------------------------------
//
//-------------------------------------------------------------------
//
//
//
//
//

string sTfTable[] = {"M1","M5","M15","M30","H1","H4","D1","W1","MN"};
int    iTfTable[] = {1,5,15,30,60,240,1440,10080,43200};

string timeFrameToString(int tf)
{
   for (int i=ArraySize(iTfTable)-1; i>=0; i--) 
         if (tf==iTfTable[i]) return(sTfTable[i]);
                              return("");
}